@@ -1,4 +1,4 @@
 from .geometry import Geometry
 from .material import Material
-from .solver import Solver
+from .shared import Solver
 from .plot import Plot
